/*

  in_cube Gamecube Stream Player for Winamp
  by hcs

  includes work by Destop and bero

*/

// AHX (MPEG2 layer 2?)
#include <windows.h>
#include "cube.h"
#include "wamain.h"

// inputfile == NULL means file is already opened, just reload
// return 1 if valid AHX not detected, 0 on success
int InitAHXFILE(char * inputfile, CUBEFILE * ahx) {
	unsigned char readbuffer[4096],*preadbuf;
	int l;
	int offs;

	if (inputfile) {
		ahx->ch[0].infile=ahx->ch[1].infile=INVALID_HANDLE_VALUE;
		ahx->ch[0].infile = CreateFile(inputfile,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,
			OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);

		if (ahx->ch[0].infile == INVALID_HANDLE_VALUE) // error opening file
		{
			return 1;
		}
	}

	SetFilePointer(ahx->ch[0].infile,0,0,FILE_BEGIN);
	
	ReadFile(ahx->ch[0].infile, readbuffer, 4096, &l, NULL);

	if(readbuffer[0] != 0x80) 
	{
			// not AHX
			if (inputfile) {
				CloseHandle(ahx->ch[0].infile);
				ahx->ch[0].infile=INVALID_HANDLE_VALUE;
			}
			return 1;
	} else preadbuf=readbuffer+(offs=0);

	ahx->ch[0].chanstart = get16bit(&preadbuf[2])+4;
	ahx->ch[1].chanstart = get16bit(&preadbuf[2])+4+18;

 	if(ahx->ch[0].chanstart < 0 || ahx->ch[0].chanstart > 4096 || memcmp(preadbuf+ahx->ch[0].chanstart-6,"(c)CRI",6)) 
	{
		// not AHX
		if (inputfile) {
			CloseHandle(ahx->ch[0].infile);
			ahx->ch[0].infile=INVALID_HANDLE_VALUE;
		}
		return 1;
	}

	ahx->ADXCH = ahx->NCH = (int)preadbuf[7];

	if (ahx->NCH>2) {
		if (inputfile) {
			CloseHandle(ahx->ch[0].infile);
			ahx->ch[0].infile=INVALID_HANDLE_VALUE;
		}
		return 1;
	}
	ahx->ch[0].sample_rate = get32bit(&preadbuf[8]);
	ahx->nrsamples = get32bit(&preadbuf[12]);
		
	// check version code, set up looping appropriately
	if (get32bit(&preadbuf[0x10])==0x00000600) { // AHX (Shenmue)
			ahx->ch[0].loop_flag=0;
	} else {
		if (inputfile) {
			CloseHandle(adx->ch[0].infile);
			ahx->ch[0].infile=INVALID_HANDLE_VALUE;
		}
		return 1;
	}

	ahx->ch[0].sa+=offs;
	ahx->ch[0].ea+=offs;
	ahx->ch[0].chanstart+=offs;
	ahx->ch[1].chanstart+=offs;

	if (ahx->ch[0].loop_flag) ahx->nrsamples=((ahx->ch[0].sa-ahx->ch[0].chanstart)+(ahx->ch[0].ea-ahx->ch[0].sa)*looptimes)*32/ahx->NCH/18+(fadelength+fadedelay)*ahx->ch[0].sample_rate;

	SetFilePointer(ahx->ch[0].infile, ahx->ch[0].chanstart, NULL, FILE_BEGIN);
	
    ahx->file_length=GetFileSize(ahx->ch[0].infile,NULL);

	ahx->ch[0].readloc=ahx->ch[1].readloc=ahx->ch[0].writeloc=ahx->ch[1].writeloc=0;

	return 0;
}

void fillbufferAHX(CUBEFILE * ahx) {
	int i,j,l;
	short decodebuf[32];
	char ADPCMbuf[18];

	if ((signed long)SetFilePointer(adx->ch[0].infile,0,0,FILE_CURRENT) >= adx->file_length && !adx->ch[0].loop_flag) {
		adx->ch[0].readloc=adx->ch[1].readloc=adx->ch[0].writeloc-1;
		return;
	}

	do {
		if (adx->ch[0].loop_flag && SetFilePointer(adx->ch[0].infile,0,0,FILE_CURRENT) >= adx->ch[0].ea) {
			//DisplayError("loop");
			SetFilePointer(adx->ch[0].infile,adx->ch[0].sa,0,FILE_BEGIN);
		}

		l = 0;

		for (j=0;j<adx->ADXCH;j++) {
			if (adxonechan && j+1!=adxonechan) SetFilePointer(adx->ch[0].infile,18,0,FILE_CURRENT);
			else {
				ReadFile(adx->ch[0].infile, ADPCMbuf, 18, &l, NULL);
				if (l<18) return;

				if (adxonechan) {
					ADXdecodebuffer(ADPCMbuf,decodebuf, &adx->ch[0].hist1, &adx->ch[0].hist2);
					for(i = 0; i < 32; i++)
						adx->ch[0].chanbuf[adx->ch[0].writeloc+i] = decodebuf[i];
				} else {
					ADXdecodebuffer(ADPCMbuf,decodebuf, &adx->ch[j].hist1, &adx->ch[j].hist2);
					for(i = 0; i < 32; i++)
						adx->ch[j].chanbuf[adx->ch[j].writeloc+i] = decodebuf[i];
				}
			}
		}
		adx->ch[0].writeloc+=32;
		if (adx->ch[0].writeloc>=0x8000/8*14) adx->ch[0].writeloc=0;
		
		if (adx->NCH==2) {
			adx->ch[1].writeloc+=32;
			if (adx->ch[1].writeloc>=0x8000/8*14) adx->ch[1].writeloc=0;
		}
	} while (adx->ch[0].writeloc != adx->ch[0].readloc);
} 

